package tomp.dyngraphs;

import tomp.graphs.GraphException;

import java.util.*;

/**
 * The <code>DynGraph</code> is an implementation of the dynamic graph, allowing
 * to add/remove/query its nodes,
 * and add/remove/query edges between its nodes.
 */
public class DynGraph implements Graph {

    /**
     * The Map <code>nodes</code> stores pairs (key=Node -&gt; value=Set of Nodes
     * - the neighbors of the node).
     */
    protected Map nodes;

    public DynGraph() {
        nodes = new HashMap();
    }

    /**
     * @return Set of all nodes in this graph.
     */
    public Set nodes() {
        return nodes.keySet();
    }

    /**
     * Adds a Node <code>node</code> into the graph.
     * The inserted Node will have no neighbors, i.e. no outgoing edges from it.
     */
    public void addNode(Node node) {
        nodes.put(node, new HashSet());
    }

    /**
     * Tests whether this graph contains the Node <code>node</code>.
     * @param node A Node
     * @return Returns <code>true</code> if and only if
     * this graph contains the Node <code>node</code>.
     */
    public boolean hasNode(Node node) {
        return nodes.containsKey(node);
    }

    /**
     * Removes the Node <code>node</code> from the graph.
     * @param node A Node to be removed.
     * @exception GraphException is thrown
     * iff the provided node is not present in this graph.
     */
    public void removeNode(Node node) throws GraphException {
        if (hasNode(node)) {
            nodes.remove(node);
        } else {
            throw new GraphException("The graph does not have the node "+node.getName());
        }
    }

    /**
     * Returns the neighbors of the Node <code>node</code> in the graph.
     * @param node A Node
     * @return Returns the nodes that are neighbors of the provided <code>node</code>.
     * @exception GraphException is thrown
     * if the <code>node</code> is not present in this graph.
     */
    public Set neighbors(Node node) throws GraphException {
        if (hasNode(node)) {
            return (Set)nodes.get(node);
        } else {
            throw new GraphException("The graph does not have the node "+node.getName());
        }
    }

    /**
     * Tests whether this graph has an edge between nodes <code>start</code> and <code>end</code>.
     * @param start The starting (source, head) node of the edge
     * @param end The ending (destination, tail) node of the edge
     * @return Returns <code>true</code> iff this graph
     * has an edge between nodes <code>start</code> and <code>end</code>.
     * @exception GraphException is thrown
     * if either the <code>start</code> or <code>end</code> are not present in this graph.
     */
    public boolean hasEdge(Node start, Node end) throws GraphException {
        ensureHasBothNodes(start, end);
        return neighbors(start).contains(end);
    }

    /**
     * Adds an edge between nodes <code>start</code> and <code>end</code>.
     * @param start The starting (source, head) node of the edge
     * @param end The ending (destination, tail) node of the edge
     * @exception GraphException is thrown
     * if either the <code>start</code> or <code>end</code> are not present in this graph.
     */
    public void setEdge(Node start, Node end) throws GraphException {
        ensureHasBothNodes(start, end);
        neighbors(start).add(end);
    }

    /**
     * Removes the edge between nodes <code>start</code> and <code>end</code>.
     * @param start The starting (source, head) node of the edge
     * @param end The ending (destination, tail) node of the edge
     * @exception GraphException is thrown
     * if either the <code>start</code> or <code>end</code> are not present in this graph
     * or if the removed edge is not in the graph.
     */
    public void removeEdge(Node start, Node end) throws GraphException {
        ensureHasBothNodes(start, end);
        Set startNeighbors = neighbors(start);
        if (startNeighbors.contains(end)) {
            startNeighbors.remove(end);
        } else {
            throw new GraphException("The graph does not have an edge between nodes "+start.getName()
                + " and "+end.getName());
        }
    }

    /**
     * Utility method to ensure if both nodes are in the graph.
     * If at least one of them is not in this graph, an exception is thrown.
     * @exception GraphException is thrown
     * if either the <code>start</code> or <code>end</code> are not present in this graph.
     */
    protected void ensureHasBothNodes(Node start, Node end) throws GraphException {
        if (!hasNode(start)) {
            throw new GraphException(
                "The graph does not have the starting "+start.getName());
        }
        if (!hasNode(end)) {
            throw new GraphException(
                "The graph does not have the ending node "+end.getName());
        }
    }
}